@@ -1,4 +1,4 @@
-CREATE OR REPLACE FUNCTION @extschema@.partition_data_time(
+CREATE FUNCTION @extschema@.partition_data_time(
     p_parent_table text
     , p_batch_count int DEFAULT 1
     , p_batch_interval interval DEFAULT NULL
@@ -131,6 +131,7 @@ SELECT partition_tablename INTO v_last_partition FROM @extschema@.show_partition
 v_partition_expression := CASE
     WHEN v_epoch = 'seconds' THEN format('to_timestamp(%I)', v_control)
     WHEN v_epoch = 'milliseconds' THEN format('to_timestamp((%I/1000)::float)', v_control)
+    WHEN v_epoch = 'microseconds' THEN format('to_timestamp((%I/1000000)::float)', v_control)
     WHEN v_epoch = 'nanoseconds' THEN format('to_timestamp((%I/1000000000)::float)', v_control)
     ELSE format('%I', v_control)
 END;
@@ -161,7 +162,7 @@ FOR i IN 1..p_batch_count LOOP
         EXIT;
     END IF;
 
-    SELECT child_start_time INTO v_min_partition_timestamp FROM @extschema@.show_partition_info(v_source_schemaname||'.'||v_last_partition
+    SELECT child_start_time INTO v_min_partition_timestamp FROM @extschema@.show_partition_info(v_parent_schema||'.'||v_last_partition
         , v_partition_interval::text
         , p_parent_table);
     v_max_partition_timestamp := v_min_partition_timestamp + v_partition_interval;
