@@ -10,6 +10,7 @@ CREATE FUNCTION @extschema@.show_partitions (
 DECLARE
 
 v_control               text;
+v_time_decoder          text;
 v_control_type          text;
 v_exact_control_type    text;
 v_datetime_string       text;
@@ -35,10 +36,12 @@ END IF;
 SELECT partition_type
     , datetime_string
     , control
+    , time_decoder
     , epoch
 INTO v_partition_type
     , v_datetime_string
     , v_control
+    , v_time_decoder
     , v_epoch
 FROM @extschema@.part_config
 WHERE parent_table = p_parent_table;
@@ -75,6 +78,7 @@ v_sql := format('SELECT n.nspname::text AS partition_schemaname
     , v_parent_schema
     , v_parent_tablename);
 
+
 IF p_include_default THEN
     -- Return the default partition immediately as first item in list
     v_default_sql := v_sql || format('
@@ -86,19 +90,27 @@ END IF;
 v_sql := v_sql || format('
     AND pg_get_expr(relpartbound, c.oid) != ''DEFAULT'' ');
 
-
 IF v_control_type = 'time' THEN
 
     v_sql := v_sql || format('
         ORDER BY (regexp_match(pg_get_expr(c.relpartbound, c.oid, true), $REGEX$\(([^)]+)\) TO \(([^)]+)\)$REGEX$))[1]::text::timestamptz %s '
         , p_order);
 
+ELSIF v_control_type IN ('text', 'uuid') THEN
+
+    v_sql := v_sql || format('
+        ORDER BY %s((regexp_match(pg_get_expr(c.relpartbound, c.oid, true), $REGEX$\(''([^)]+)''\) TO \(''([^)]+)''\)$REGEX$))[1]) %s '
+        , v_time_decoder
+        , p_order);
+
 ELSIF v_control_type = 'id' AND v_epoch <> 'none' THEN
 
     IF v_epoch = 'seconds' THEN
         v_epoch_divisor := 1;
     ELSIF v_epoch = 'milliseconds' THEN
         v_epoch_divisor := 1000;
+    ELSIF v_epoch = 'microseconds' THEN
+        v_epoch_divisor := 1000000;
     ELSIF v_epoch = 'nanoseconds' THEN
         v_epoch_divisor := 1000000000;
     END IF;
@@ -112,13 +124,13 @@ ELSIF v_control_type = 'id' AND v_epoch <> 'none' THEN
 ELSIF v_control_type = 'id' THEN
 
     IF v_partition_type = 'range' THEN
-        -- Have to do a trim here because of inconsistency in quoting different integer types. Ex: bigint boundary values are quoted but int values are not
+        -- Have to do trims here because of inconsistency in quoting different integer types. Ex: bigint boundary values are quoted but int values are not
         v_sql := v_sql || format('
             ORDER BY trim( BOTH $QUOTE$''$QUOTE$ from (regexp_match(pg_get_expr(c.relpartbound, c.oid, true), $REGEX$\(([^)]+)\) TO \(([^)]+)\)$REGEX$))[1]::text )::%s %s '
             , v_exact_control_type, p_order);
     ELSIF v_partition_type = 'list' THEN
         v_sql := v_sql || format('
-            ORDER BY trim((regexp_match(pg_get_expr(c.relpartbound, c.oid, true), $REGEX$FOR VALUES IN \(([^)])\)$REGEX$))[1])::%s %s '
+            ORDER BY trim( BOTH $QUOTES$''$QUOTES$ from (regexp_match(pg_get_expr(c.relpartbound, c.oid, true), $REGEX$FOR VALUES IN \(([^)]+)\)$REGEX$))[1])::%s %s '
             , v_exact_control_type , p_order);
     ELSE
         RAISE EXCEPTION 'show_partitions: Unsupported partition type found: %', v_partition_type;
