@@ -1,6 +1,8 @@
 CREATE TABLE @extschema@.part_config (
     parent_table text NOT NULL
     , control text NOT NULL
+    , time_encoder text
+    , time_decoder text
     , partition_interval text NOT NULL
     , partition_type text NOT NULL
     , premake int NOT NULL DEFAULT 4
@@ -21,7 +23,6 @@ CREATE TABLE @extschema@.part_config (
     , inherit_privileges boolean DEFAULT false
     , constraint_valid boolean DEFAULT true NOT NULL
     , ignore_default_data boolean NOT NULL DEFAULT true
-    , default_table boolean DEFAULT true
     , date_trunc_interval text
     , maintenance_order int
     , retention_keep_publication boolean NOT NULL DEFAULT false
@@ -38,6 +39,8 @@ SELECT pg_catalog.pg_extension_config_dump('@extschema@.part_config'::regclass,
 CREATE TABLE @extschema@.part_config_sub (
     sub_parent text
     , sub_control text NOT NULL
+    , sub_time_encoder text
+    , sub_time_decoder text
     , sub_partition_interval text NOT NULL
     , sub_partition_type text NOT NULL
     , sub_premake int NOT NULL DEFAULT 4
@@ -59,6 +62,7 @@ CREATE TABLE @extschema@.part_config_sub (
     , sub_date_trunc_interval TEXT
     , sub_maintenance_order int
     , sub_retention_keep_publication boolean NOT NULL DEFAULT false
+    , sub_control_not_null boolean DEFAULT true
     , CONSTRAINT part_config_sub_pkey PRIMARY KEY (sub_parent)
     , CONSTRAINT part_config_sub_sub_parent_fkey FOREIGN KEY (sub_parent) REFERENCES @extschema@.part_config (parent_table) ON DELETE CASCADE ON UPDATE CASCADE DEFERRABLE INITIALLY DEFERRED
     , CONSTRAINT positive_premake_check CHECK (sub_premake > 0)
@@ -131,13 +135,13 @@ CHECK (@extschema@.check_automatic_maintenance_value(sub_automatic_maintenance))
  * Check function for config table epoch types
  */
 CREATE FUNCTION @extschema@.check_epoch_type (p_type text) RETURNS boolean
-    LANGUAGE plpgsql IMMUTABLE SECURITY DEFINER
+    LANGUAGE plpgsql IMMUTABLE
     SET search_path TO pg_catalog, pg_temp
     AS $$
 DECLARE
 v_result    boolean;
 BEGIN
-    SELECT p_type IN ('none', 'seconds', 'milliseconds', 'nanoseconds') INTO v_result;
+    SELECT p_type IN ('none', 'seconds', 'milliseconds', 'microseconds', 'nanoseconds') INTO v_result;
     RETURN v_result;
 END
 $$;
@@ -157,7 +161,7 @@ CHECK (@extschema@.check_epoch_type(sub_epoch));
  */
 -- Allow hash in future update
 CREATE FUNCTION @extschema@.check_partition_type (p_type text) RETURNS boolean
-    LANGUAGE plpgsql IMMUTABLE SECURITY DEFINER
+    LANGUAGE plpgsql IMMUTABLE
     SET search_path TO pg_catalog, pg_temp
     AS $$
 DECLARE
