@@ -28,10 +28,9 @@
 #include "utils/snapmgr.h"
 #include "tcop/utility.h"
 #include "commands/async.h"
-
-#if (PG_VERSION_NUM >= 100000)
 #include "utils/varlena.h"
-#endif
+#include "tcop/pquery.h"
+#include "utils/memutils.h"
 
 
 PG_MODULE_MAGIC;
@@ -46,23 +45,16 @@ static volatile sig_atomic_t got_sigterm = false;
 
 /* GUC variables */
 static int pg_partman_bgw_interval = 3600; // Default hourly
+static int pg_partman_bgw_maintenance_wait = 0; // Default no wait
 static char *pg_partman_bgw_role = "postgres"; // Default to postgres role
 
-// Do not analyze by default on PG11+
-#if (PG_VERSION_NUM >= 110000)
+// Do not analyze by default
 static char *pg_partman_bgw_analyze = "off";
-#else
-static char *pg_partman_bgw_analyze = "on";
-#endif
 
 static char *pg_partman_bgw_jobmon = "on";
 static char *pg_partman_bgw_dbname = NULL;
 
-#if (PG_VERSION_NUM < 100500)
-static bool (*split_function_ptr)(char *, char, List **) = &SplitIdentifierString;
-#else
 static bool (*split_function_ptr)(char *, char, List **) = &SplitGUCList;
-#endif
 
 /*
  * Signal handler for SIGTERM
@@ -119,7 +111,19 @@ _PG_init(void)
                             NULL,
                             NULL);
 
-    #if (PG_VERSION_NUM >= 110000)
+    DefineCustomIntVariable("pg_partman_bgw.maintenance_wait",
+                            "How long to wait between each partition set when running maintenance (in seconds).",
+                            NULL,
+                            &pg_partman_bgw_maintenance_wait,
+                            0,
+                            0,
+                            INT_MAX,
+                            PGC_SIGHUP,
+                            0,
+                            NULL,
+                            NULL,
+                            NULL);
+
     DefineCustomStringVariable("pg_partman_bgw.analyze",
                             "Whether to run an analyze on a partition set whenever a new partition is created during run_maintenance(). Set to 'on' to send TRUE (default). Set to 'off' to send FALSE.",
                             NULL,
@@ -130,18 +134,6 @@ _PG_init(void)
                             NULL,
                             NULL,
                             NULL);
-    #else
-    DefineCustomStringVariable("pg_partman_bgw.analyze",
-                            "Whether to run an analyze on a partition set whenever a new partition is created during run_maintenance(). Set to 'on' to send TRUE (default). Set to 'off' to send FALSE.",
-                            NULL,
-                            &pg_partman_bgw_analyze,
-                            "on",
-                            PGC_SIGHUP,
-                            0,
-                            NULL,
-                            NULL,
-                            NULL);
-    #endif  
 
     DefineCustomStringVariable("pg_partman_bgw.dbname",
                             "CSV list of specific databases in the cluster to run pg_partman BGW on.",
@@ -194,17 +186,13 @@ _PG_init(void)
 
     // Start BGW when database starts
     sprintf(worker.bgw_name, "pg_partman master background worker");
+    sprintf(worker.bgw_type, "pg_partman background worker");
     worker.bgw_flags = BGWORKER_SHMEM_ACCESS |
         BGWORKER_BACKEND_DATABASE_CONNECTION;
     worker.bgw_start_time = BgWorkerStart_RecoveryFinished;
     worker.bgw_restart_time = 600;
-    #if (PG_VERSION_NUM < 100000)
-    worker.bgw_main = pg_partman_bgw_main;
-    #endif
-    #if (PG_VERSION_NUM >= 100000)
     sprintf(worker.bgw_library_name, "pg_partman_bgw");
     sprintf(worker.bgw_function_name, "pg_partman_bgw_main");
-    #endif
     worker.bgw_main_arg = CStringGetDatum(pg_partman_bgw_dbname);
     worker.bgw_notify_pid = 0;
     RegisterBackgroundWorker(&worker);
@@ -243,7 +231,7 @@ void pg_partman_bgw_main(Datum main_arg) {
         char                    *rawstring;
         int                     dbcounter;
         int                     rc;
-        int                     full_string_length;
+        size_t                  full_string_length;
         List                    *elemlist;
         ListCell                *l;
         pid_t                   pid;
@@ -267,13 +255,13 @@ void pg_partman_bgw_main(Datum main_arg) {
             return;
         }
 
-        // Use method of shared_preload_libraries to split the pg_partman_bgw_dbname string found in src/backend/utils/init/miscinit.c 
-        // Need a modifiable copy of string 
+        // Use method of shared_preload_libraries to split the pg_partman_bgw_dbname string found in src/backend/utils/init/miscinit.c
+        // Need a modifiable copy of string
         if (pg_partman_bgw_dbname != NULL) {
             rawstring = pstrdup(pg_partman_bgw_dbname);
-            // Parse string into list of identifiers 
+            // Parse string into list of identifiers
             if (!(*split_function_ptr)(rawstring, ',', &elemlist)) {
-                // syntax error in list 
+                // syntax error in list
                 pfree(rawstring);
                 list_free(elemlist);
                 ereport(LOG,
@@ -281,20 +269,17 @@ void pg_partman_bgw_main(Datum main_arg) {
                          errmsg("invalid list syntax in parameter \"pg_partman_bgw.dbname\" in postgresql.conf")));
                 return;
             }
-            
+
             dbcounter = 0;
             foreach(l, elemlist) {
 
                 char *dbname = (char *) lfirst(l);
-                
+
                 elog(DEBUG1, "Dynamic bgw launch begun for %s (%d)", dbname, dbcounter);
                 worker.bgw_flags = BGWORKER_SHMEM_ACCESS |
                     BGWORKER_BACKEND_DATABASE_CONNECTION;
                 worker.bgw_start_time = BgWorkerStart_RecoveryFinished;
                 worker.bgw_restart_time = BGW_NEVER_RESTART;
-                #if (PG_VERSION_NUM < 100000)
-                worker.bgw_main = NULL;
-                #endif
                 sprintf(worker.bgw_library_name, "pg_partman_bgw");
                 sprintf(worker.bgw_function_name, "pg_partman_bgw_run_maint");
                 full_string_length = snprintf(worker.bgw_name, sizeof(worker.bgw_name),
@@ -305,6 +290,7 @@ void pg_partman_bgw_main(Datum main_arg) {
                     memcpy(worker.bgw_name + sizeof(worker.bgw_name) - sizeof(truncated_mark),
                            truncated_mark, sizeof(truncated_mark));
                 }
+                sprintf(worker.bgw_type, "pg_partman background worker");
                 worker.bgw_main_arg = Int32GetDatum(dbcounter);
                 worker.bgw_notify_pid = MyProcPid;
 
@@ -334,14 +320,12 @@ void pg_partman_bgw_main(Datum main_arg) {
                 }
                 Assert(status == BGWH_STARTED);
 
-                #if (PG_VERSION_NUM >= 90500)
-                // Shutdown wait function introduced in 9.5. The latch problems this wait fixes are only encountered in 
-                // 9.6 and later. So this shouldn't be a problem for 9.4.
+                // Shutdown wait function introduced in 9.5. The latch problems this wait fixes are only encountered in
+                // 9.6 and later.
                 elog(DEBUG1, "Waiting for BGW shutdown...");
                 status = WaitForBackgroundWorkerShutdown(handle);
                 elog(DEBUG1, "BGW shutdown status: %d", status);
                 Assert(status == BGWH_STOPPED);
-                #endif
             }
 
             pfree(rawstring);
@@ -353,17 +337,10 @@ void pg_partman_bgw_main(Datum main_arg) {
 
         elog(DEBUG1, "Latch status just before waitlatch call: %d", MyProc->procLatch.is_set);
 
-        #if (PG_VERSION_NUM >= 100000)
         rc = WaitLatch(&MyProc->procLatch,
                        WL_LATCH_SET | WL_TIMEOUT | WL_POSTMASTER_DEATH,
                        pg_partman_bgw_interval * 1000L,
                        PG_WAIT_EXTENSION);
-        #endif
-        #if (PG_VERSION_NUM < 100000)
-        rc = WaitLatch(&MyProc->procLatch,
-                       WL_LATCH_SET | WL_TIMEOUT | WL_POSTMASTER_DEATH,
-                       pg_partman_bgw_interval * 1000L);
-        #endif
         /* emergency bailout if postmaster has died */
         if (rc & WL_POSTMASTER_DEATH) {
             proc_exit(1);
@@ -377,7 +354,7 @@ void pg_partman_bgw_main(Datum main_arg) {
 
 /*
  * Unable to pass the database name as a string argument (not sure why yet)
- * Instead, the GUC is parsed both in the main function and below and a counter integer 
+ * Instead, the GUC is parsed both in the main function and below and a counter integer
  *  is passed to determine which database the BGW will run in.
  */
 void pg_partman_bgw_run_maint(Datum arg) {
@@ -391,6 +368,9 @@ void pg_partman_bgw_run_maint(Datum arg) {
     List                *elemlist;
     int                 ret;
     StringInfoData      buf;
+    SPIExecuteOptions   spi_exec_opts;
+    Portal              portal = ActivePortal;
+    bool                portal_created = false;
 
     /* Establish signal handlers before unblocking signals. */
     pqsignal(SIGHUP, pg_partman_bgw_sighup);
@@ -402,9 +382,9 @@ void pg_partman_bgw_run_maint(Datum arg) {
     elog(DEBUG1, "Before parsing dbname GUC in dynamic main func: %s", pg_partman_bgw_dbname);
     rawstring = pstrdup(pg_partman_bgw_dbname);
     elog(DEBUG1, "GUC rawstring copy: %s", rawstring);
-    // Parse string into list of identifiers 
+    // Parse string into list of identifiers
     if (!(*split_function_ptr)(rawstring, ',', &elemlist)) {
-        // syntax error in list 
+        // syntax error in list
         pfree(rawstring);
         list_free(elemlist);
         ereport(LOG,
@@ -415,28 +395,33 @@ void pg_partman_bgw_run_maint(Datum arg) {
 
     dbname = list_nth(elemlist, db_main_counter);
     elog(DEBUG1, "Parsing dbname list: %s (%d)", dbname, db_main_counter);
-    
+
     if (strcmp(dbname, "template1") == 0) {
         elog(DEBUG1, "Default database name found in dbname local variable (\"template1\").");
     }
 
     elog(DEBUG1, "Before run_maint initialize connection for db %s", dbname);
 
-    #if (PG_VERSION_NUM < 110000)
-    BackgroundWorkerInitializeConnection(dbname, pg_partman_bgw_role);
-    #endif
-    #if (PG_VERSION_NUM >= 110000)
     BackgroundWorkerInitializeConnection(dbname, pg_partman_bgw_role, 0);
-    #endif
-    
+
     elog(DEBUG1, "After run_maint initialize connection for db %s", dbname);
 
     initStringInfo(&buf);
 
     SetCurrentStatementStartTimestamp();
-    StartTransactionCommand();
-    SPI_connect();
-    PushActiveSnapshot(GetTransactionSnapshot());
+
+    SPI_connect_ext(SPI_OPT_NONATOMIC);
+    if (!PortalIsValid(portal)) {
+        portal_created = true;
+        portal = CreateNewPortal();
+        portal->visible = false;
+        portal->resowner = CurrentResourceOwner;
+        ActivePortal = portal;
+        PortalContext = portal->portalContext;
+        StartTransactionCommand();
+        EnsurePortalSnapshotExists();
+    }
+
     pgstat_report_appname("pg_partman dynamic background worker");
 
     // First determine if pg_partman is even installed in this database
@@ -461,14 +446,14 @@ void pg_partman_bgw_run_maint(Datum arg) {
         return;
     }
 
-    // If so then actually log that it's started for that database. 
+    // If so then actually log that it's started for that database.
     elog(LOG, "%s dynamic background worker initialized with role %s on database %s"
             , MyBgworkerEntry->bgw_name
             , pg_partman_bgw_role
             , dbname);
 
     resetStringInfo(&buf);
-    appendStringInfo(&buf, "SELECT n.nspname FROM pg_catalog.pg_extension e JOIN pg_catalog.pg_namespace n ON e.extnamespace = n.oid WHERE extname = 'pg_partman'");
+    appendStringInfo(&buf, "SELECT pg_catalog.quote_ident(n.nspname) FROM pg_catalog.pg_extension e JOIN pg_catalog.pg_namespace n ON e.extnamespace = n.oid WHERE e.extname = 'pg_partman'");
     pgstat_report_activity(STATE_RUNNING, buf.data);
     ret = SPI_execute(buf.data, true, 1);
 
@@ -479,11 +464,13 @@ void pg_partman_bgw_run_maint(Datum arg) {
     if (SPI_processed > 0) {
         bool isnull;
 
-        partman_schema = DatumGetCString(SPI_getbinval(SPI_tuptable->vals[0]
+        partman_schema = TextDatumGetCString(SPI_getbinval(SPI_tuptable->vals[0]
                 , SPI_tuptable->tupdesc
                 , 1
                 , &isnull));
 
+        elog(DEBUG1, "pg_partman_bgw: pg_partman schema: %s.", partman_schema);
+
         if (isnull)
             elog(FATAL, "Query to determine pg_partman schema returned NULL.");
 
@@ -502,14 +489,19 @@ void pg_partman_bgw_run_maint(Datum arg) {
     } else {
         jobmon = "false";
     }
-    appendStringInfo(&buf, "SELECT \"%s\".run_maintenance(p_analyze := %s, p_jobmon := %s)", partman_schema, analyze, jobmon);
+
+    appendStringInfo(&buf, "CALL %s.run_maintenance_proc(p_wait => %d, p_analyze => %s, p_jobmon => %s)", partman_schema, pg_partman_bgw_maintenance_wait, analyze, jobmon);
 
     pgstat_report_activity(STATE_RUNNING, buf.data);
 
-    ret = SPI_execute(buf.data, false, 0);
 
-    if (ret != SPI_OK_SELECT)
-        elog(FATAL, "Cannot call pg_partman run_maintenance() function: error code %d", ret);
+    // Call refresh_metric_views procedure non-atomically
+    memset(&spi_exec_opts, 0, sizeof(spi_exec_opts));
+    spi_exec_opts.allow_nonatomic = true;
+    ret = SPI_execute_extended(buf.data, &spi_exec_opts);
+
+    if (ret != SPI_OK_UTILITY)
+        elog(FATAL, "Cannot call pg_partman run_maintenance_proc() procedure: error code %d", ret);
 
     elog(LOG, "%s: %s called by role %s on database %s"
             , MyBgworkerEntry->bgw_name
@@ -518,8 +510,16 @@ void pg_partman_bgw_run_maint(Datum arg) {
             , dbname);
 
     SPI_finish();
-    PopActiveSnapshot();
-    CommitTransactionCommand();
+
+    if (portal_created) {
+        if (ActiveSnapshotSet())
+            PopActiveSnapshot();
+        CommitTransactionCommand();
+        PortalDrop(portal, false);
+        ActivePortal = NULL;
+        PortalContext = NULL;
+    }
+
     #if (PG_VERSION_NUM < 150000)
     ProcessCompletedNotifies();
     #endif
@@ -531,4 +531,3 @@ void pg_partman_bgw_run_maint(Datum arg) {
 
     return;
 }
-
